// $Id: DetectorConstruction.cc 94 2010-01-26 13:18:30Z adotti $
/**
 * @file
 * @brief Implements mandatory user class DetectorConstruction.
 */

#include "DetectorConstruction.hh"
//#include "DetectorMessenger.hh"

#include "G4Material.hh"
#include "G4Box.hh"
#include "G4Tubs.hh"
#include "G4LogicalVolume.hh"
#include "G4PVPlacement.hh"
#include "G4PVReplica.hh"

#include "G4GeometryTolerance.hh"
#include "G4GeometryManager.hh"
#include "G4NistManager.hh"

#include "G4VisAttributes.hh"
#include "G4Colour.hh"

#include "G4SDManager.hh"

DetectorConstruction::DetectorConstruction()
{
	//--------- Material definition ---------
	DefineMaterials();

	//--------- Sizes of the principal geometrical components (solids)  ---------
	ComputeParameters();
}
 
DetectorConstruction::~DetectorConstruction()
{
}
 
void DetectorConstruction::DefineMaterials() 
{
	//Get Materials from NIST database
	G4NistManager* man = G4NistManager::Instance();
	man->SetVerbose(0);

	// define NIST materials
	vacuum  = man->FindOrBuildMaterial("G4_Galactic");

	water = man->FindOrBuildMaterial("G4_WATER");

	lead = man->FindOrBuildMaterial("G4_Pb");

	air     = man->FindOrBuildMaterial("G4_AIR");

}
 
void DetectorConstruction::ComputeParameters() 
{
	//This function defines the defaults
	//of the geometry construction

	// ** world **
	halfWorldLength = 5* m;

}
 
G4VPhysicalVolume* DetectorConstruction::Construct()
{
	//This function is called by G4 when the detector has to be created
	//--------- Definitions of Solids, Logical Volumes, Physical Volumes ---------

  
	//------------------------------
	// World
	//------------------------------
 
	G4GeometryManager::GetInstance()->SetWorldMaximumExtent(2.*halfWorldLength);
	G4cout << "Computed tolerance = "
			<< G4GeometryTolerance::GetInstance()->GetSurfaceTolerance()/mm
			<< " mm" << G4endl;


	G4Box * solidWorld= new G4Box("world",halfWorldLength,halfWorldLength,halfWorldLength);
	logicWorld= new G4LogicalVolume( solidWorld, air, "World", 0, 0, 0);
  
	//  Must place the World Physical volume unrotated at (0,0,0).
	//
	G4VPhysicalVolume * physiWorld = new G4PVPlacement(0,               // no rotation
			G4ThreeVector(), // at (0,0,0)
			logicWorld,      // its logical volume
			"World",         // its name
			0,               // its mother  volume
			false,           // no boolean operations
			0);              // copy number
				 


	//Construction of the three si plane is actually done here
	ConstructCollimator();

	//Construction of the EM calorimeter
	ConstructBox();


	//--------- Visualization attributes -------------------------------

	G4Color
		green(0.0,1.0,0.0),
		blue(0.0,0.0,1.0),
		brown(0.4,0.4,0.1),
		white(1.0,1.0,1.0);

	logicWorld -> SetVisAttributes(new G4VisAttributes(white));
	logicWorld -> SetVisAttributes(G4VisAttributes::Invisible);
    
	//always return the physical World
	//
	return physiWorld;
}

G4VPhysicalVolume* DetectorConstruction::ConstructCollimator()
{
	//
	// Construct lead cylindrical collimator
	//


  //Dimension collimator (beam line)
  G4double halfHadCollimatorSizeZ = 10*cm;
  G4double innerRadiusCollimator = 1*cm;
  G4double outerRadiusCollimator = 10*cm;
  posCollimator = G4ThreeVector(0, 0, 30*cm);

  
  //Create the main volume: a cylinder
  G4Tubs* collimatorSolid = new G4Tubs("collimatorSolid",//its name
				     innerRadiusCollimator, //inner radius
				     outerRadiusCollimator,//outer radius
				     halfHadCollimatorSizeZ, //half length in z
				     0, //Starting angle in phi
				     360*deg);//Ending angle in phi
  
  //Create the main volume logical volume.
  //A logical volume is made of: a solid and a material
  //A name must also be given
  G4LogicalVolume* collimatorLogic = new G4LogicalVolume(collimatorSolid,//its solid
						       lead,//its material
						       "collimatorLogic");//its name

  //We finish now the creation of the HAD calorimeter creating a placement:
  //We place the volume in the "World"
  collimator = new G4PVPlacement( 0,              //no rotation
			       posCollimator,     //translation
			       collimatorLogic,   //its logical volume
			       "collimator",      //its name
			       logicWorld,     //its mother volume
			       false,
			       10);//copy number




  // G4Color red(1.0,0.0,0.0),yellow(1.0,1.0,0.0);
  G4Color green(0.0,1.0,0.0);
  collimatorLogic -> SetVisAttributes(new G4VisAttributes(green));

  return collimator;

}


G4VPhysicalVolume* DetectorConstruction::ConstructBox()
{
        posBox = G4ThreeVector(0, 0, 80.*cm);
	G4double halfBoxSizeX = 20.*cm;
	G4double halfBoxSizeY = 20.*cm;
	G4double halfBoxSizeZ = 20.*cm;

	G4Box* boxSolid = new G4Box("boxSolid",halfBoxSizeX,halfBoxSizeY,halfBoxSizeZ);

	G4LogicalVolume* boxLogic = new G4LogicalVolume( boxSolid,//its solid
							 water, //its material
							 "boxLogic");//its name

	box = new G4PVPlacement(0, //no rotation
							   posBox,//translation
							   boxLogic, //its logical volume
							   "box",//its name
							   logicWorld,//its mother volume
							   false, //no boolean operation
							   11); //copy number

	G4Color brown(0.4,0.4,0.1);
	boxLogic -> SetVisAttributes(new G4VisAttributes(brown));

	return box;

}
